package com.zh.project0512.aop;

import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.Admin;
import com.zh.project0512.service.IAdminRoleService;
import com.zh.project0512.service.IAdminService;
import com.zh.project0512.utils.BizException;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.List;


/**
 * 检查权限切面类
 */
@Order(3)//设置切面优先级为3，优先级越小优先级越高
@Aspect
@Component
public class AdminCheckAuthorityAspect {

    private final static Logger logger = LoggerFactory.getLogger(AdminCheckAuthorityAspect.class);

    @Resource
    IAdminService adminService;
    @Resource
    IAdminRoleService adminRoleService;


    /**
     * 功能描述: 切点
     */
    @Pointcut("@annotation(com.zh.project0512.annotation.AdminCheckAuthorityAnnotation)")
    public void executePointCut() {

    }

    /**
     * 功能描述: 请求前执行
     */
    @Before("executePointCut()")
    public void doBefore(JoinPoint joinPoint) {
        // 接收到请求，记录请求内容
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //获取当前用户的信息
        String tokenId = request.getHeader("admin-token");
        Integer id = Integer.valueOf(new JwtUtil().parseAdminJWT(tokenId).getId());

        Admin admin = adminService.getById(id);
        //1、查询【当前用户】所拥有的【角色】
        String roleIdsStr = admin.getRoleIds();
        if (roleIdsStr != null && !roleIdsStr.trim().equals("")) {
            String[] roleIds = roleIdsStr.split(",");

            HashSet<Integer> jurisdictionIds = new HashSet<>();
            for (String roleId : roleIds) {
                //2、根据【角色】查询对应的权限
                List<Integer> jurisdictionIdList = adminRoleService.findJurisdictionByRoleId(roleId);

                //3、将各个角色的权限 合在一起 并进行 去重操作
                jurisdictionIds.addAll(jurisdictionIdList);
            }


            //4、获取该api所需权限
            MethodSignature signature = (MethodSignature) joinPoint.getSignature();
            AdminCheckAuthorityAnnotation annotation = signature.getMethod().getAnnotation(AdminCheckAuthorityAnnotation.class);
            String jurisdictionId = annotation.jurisdictionId();

            //5、判断该用户是否存在该api所需权限,如果存在，则通过，不存在则抛出提示权限不足
            if (!jurisdictionIds.contains(Integer.valueOf(jurisdictionId))) {
                throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION, "用户权限不足");
            }

        } else {
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION, "用户权限不足");
        }

    }

    /**
     * 请求后执行
     */
    @AfterReturning(returning = "ret", pointcut = "executePointCut()")
    public void doAfterReturning(Object ret) {

    }


}
